//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include <math.h>
#include "Unit1.h"
#include <clipbrd.hpp>
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TMainForm *MainForm;
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
     : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Draw(int w, int h)
{
     // Draw dials
     int CentreX = w/4, CentreY = h/2;
     int VCentreX = w*3/4, VCentreY = h/2;
     int OuterRadius = w/5, SubRadius = OuterRadius-5;
     int InnerRadius = OuterRadius/2, TextRadius = OuterRadius+15;
     int MechRadius = OuterRadius+10;
     int VernierRadius = OuterRadius-10;
     float TotalAngle = StrToFloat(PotAngleEdit->Text);
     float Angle, RadAngle;
     int StartX, StartY, EndX, EndY, TextX, TextY;

     Graphics::TBitmap *OffScreenBmp = new Graphics::TBitmap();
     OffScreenBmp->Height=MainForm->Height;
     OffScreenBmp->Width=MainForm->Width;

      // Draw circle with 60 diameter
     OffScreenBmp->Canvas->Brush->Color = clWhite;
     OffScreenBmp->Canvas->Brush->Style = bsSolid;
     Image->Repaint();
     OffScreenBmp->Canvas->FillRect(Image->ClientRect);
     OffScreenBmp->Canvas->Ellipse(CentreX-OuterRadius,CentreY-OuterRadius,CentreX+OuterRadius,CentreY+OuterRadius);
     OffScreenBmp->Canvas->Ellipse(CentreX-SubRadius,CentreY-SubRadius,CentreX+SubRadius,CentreY+SubRadius);

     // Calculate actual angles to use adjusted for pot inaccuracies
     float MinRes = StrToFloat(PotMinResEdit->Text);
     float MaxRes = StrToFloat(PotMaxResEdit->Text);
     float NomRes = StrToFloat(PotNomResEdit->Text);
     float ActualAngleStart = (-TotalAngle/2)-90 - TotalAngle*MinRes/(MaxRes-MinRes);
     float ActualAngleEnd = TotalAngle * NomRes/(MaxRes-MinRes);

     // Draw mechanical endstop markers
     if (MechLimitsCheck->Checked)
     {
          float MechStartAngle = ((-TotalAngle/2)-90) * M_PI / 180.0;
          float MechEndAngle = ((-TotalAngle/2)-90 + TotalAngle) * M_PI / 180.0;
          OffScreenBmp->Canvas->Pen->Width = 2;
          OffScreenBmp->Canvas->MoveTo(CentreX,CentreY);
          OffScreenBmp->Canvas->LineTo(CentreX + MechRadius*cos(MechStartAngle),
                                       CentreY + MechRadius*sin(MechStartAngle));
          OffScreenBmp->Canvas->MoveTo(CentreX,CentreY);
          OffScreenBmp->Canvas->LineTo(CentreX + MechRadius*cos(MechEndAngle),
                                       CentreY + MechRadius*sin(MechEndAngle));
          OffScreenBmp->Canvas->Pen->Width = 1;
     }

     // Draw Marks 0 to 10 around dial
     for (int i=0 ; i<=90 ; i++)
     {
          Angle = ActualAngleStart + i/90.0*ActualAngleEnd;
          RadAngle = Angle * M_PI / 180.0;
          StartX = CentreX + InnerRadius*cos(RadAngle);
          StartY = CentreY + InnerRadius*sin(RadAngle);
          TextX =  CentreX + TextRadius*cos(RadAngle) - OffScreenBmp->Canvas->TextWidth(AnsiString(i/10))/2;
          TextY =  CentreY + TextRadius*sin(RadAngle) - OffScreenBmp->Canvas->TextHeight(AnsiString(i/10))/2;
          if (i % 10 == 0)
          {
               EndX =   CentreX + OuterRadius*cos(RadAngle);
               EndY =   CentreY + OuterRadius*sin(RadAngle);
               if (Markings->ItemIndex == 0)
                    OffScreenBmp->Canvas->TextOut(TextX, TextY, AnsiString((i+10)/10));
               else
                    OffScreenBmp->Canvas->TextOut(TextX, TextY, AnsiString(6*(i+10)/10));
          }
          else
          {
               EndX =   CentreX + SubRadius*cos(RadAngle);
               EndY =   CentreY + SubRadius*sin(RadAngle);
          }

          OffScreenBmp->Canvas->MoveTo(StartX, StartY);
          OffScreenBmp->Canvas->LineTo(EndX, EndY);
     }


     // Draw Vernier dial
     for (int i=0 ; i<=9 ; i++)
     {
          Angle = ActualAngleStart + 0.9*i*ActualAngleEnd/100.0;
          RadAngle = Angle * M_PI / 180.0;
          EndX =   VCentreX + VernierRadius*cos(RadAngle);
          EndY =   VCentreY + VernierRadius*sin(RadAngle);

          OffScreenBmp->Canvas->MoveTo(VCentreX, VCentreY);
          OffScreenBmp->Canvas->LineTo(EndX, EndY);
     }

     OffScreenBmp->Canvas->TextOut(CentreX-OffScreenBmp->Canvas->TextWidth(Markings->Items->Strings[Markings->ItemIndex])/2,
                            CentreY+(OuterRadius*1.2),
                            Markings->Items->Strings[Markings->ItemIndex]);

     Image->Picture->Graphic = OffScreenBmp;
     delete OffScreenBmp;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormResize(TObject *Sender)
{
     // Attempt to make drawing beyond original bounds of image possible
     //Image->BoundsRect.Right = Image->ClientRect.Right-10;
     //Image->BoundsRect.Bottom = Image->ClientRect.Bottom-10;
     //OffScreenBmp->Canvas->ClipRect.Bottom = Image->BoundsRect.Bottom;
     //OffScreenBmp->Canvas->ClipRect.Right = Image->BoundsRect.Right;

     // Redraw in new scale
     Image->Width = MainForm->Width - 166;
     Image->Height = MainForm->Height - 74;
     Draw(Image->Width, Image->Height);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormCreate(TObject *Sender)
{
     MainForm->DoubleBuffered=true;
     Draw(417, 281);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::MarkingsClick(TObject *Sender)
{
     Draw(Image->Width, Image->Height);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Button1Click(TObject *Sender)
{
     Clipboard()->Assign(Image->Picture);
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::PotMinResEditKeyPress(TObject *Sender,
      char &Key)
{
     if (Key == 13)
          Draw(Image->Width, Image->Height);
}
//---------------------------------------------------------------------------

